"""The implementation of HEP Authentication protocol

"""

__copyright__ = """ Copyright (c) 2019 Newton Foundation. All rights reserved."""
__version__ = '1.0'
__author__ = 'xiawu@zeuux.org'

import logging
import datetime
from hep_rest_api.scenarios import BaseHelper
from hep_rest_api import models
from hep_rest_api import utils

logger = logging.getLogger(__name__)


class AuthHelper(BaseHelper):

    def generate_auth_request(self, scope=1, expired=300, memo='', uuid=None):
        """Generate the authentication request

        :param int scope: The request scope. 1: base profile including name,head,newid; 2: advance profile including cellphone.
        :param int expired: The expired time of request
        :param str memo: The memo of request to end user
        :param str uuid: The request uuid for desktop-browser. optional.
        :rtype: dict
        :return: The request information including signature
        """
        if not uuid:
            raise AttributeError(
                "uuid can not be None"
            )
        if not memo:
            memo = "default"
        data = {
            'action': self.action_auth_login,
            'scope': scope,
            'expired': int(datetime.datetime.now().timestamp()) + expired,
            'memo': memo,
            'uuid': uuid,
            'dapp_id': self.dapp_id
        }
        sign_data = self.generate_sign_data(data)
        hmac_data = self.sign_hmac(sign_data)
        final_data = self.sign_secp256r1(hmac_data)
        # start request
        auth_cache = models.AuthCacheRequest(**final_data)
        auth_response = self.api_client.rest_newnet_caches_auth_create(auth_cache, self.api_version)
        return auth_response

    def generate_qrcode_string(self, auth_hash):
        """Generate the hep-based scheme string string for QRCode 
        
        :param str auth_hash: The auth request hash
        :rtype: str
        :return: The QRcode string
        """
        protocol = self.base_parameters.get('protocol')
        action = self.action_auth_login
        qrcode_str = "%s://%s/?action=%s&auth_hash=%s" % (protocol.lower(), self.dapp_id, action, auth_hash)
        return qrcode_str

    def validate_auth_callback(self, data):
        """Validate the response of auth callback sent by end users such as NewPay users 
        
        :param dict data: The callback data
        :rtype: bool
        :return: True if valid data, otherwise False
        """
        newid = data.get('newid')
        signature = data.get('signature')
        signed_string = utils.generate_signature_base_string(data, "&")
        r, s = utils.split_signature(signature)
        return utils.validate_newid(r, s, signed_string, newid, self.chain_id)




# coding: utf-8

"""
    HEP REST API

    The REST API for HEP protocol  # noqa: E501

    OpenAPI spec version: v1
    Contact: xiawu@zeuux.org
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six
from hep_rest_api.models.thing import Thing  # noqa: F401,E501


class OrderItem(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'order_item_number': 'str',
        'ordered_item': 'Thing',
        'order_item_quantity': 'int',
        'price_currency': 'str',
        'price': 'str'
    }

    attribute_map = {
        'order_item_number': 'order_item_number',
        'ordered_item': 'ordered_item',
        'order_item_quantity': 'order_item_quantity',
        'price_currency': 'price_currency',
        'price': 'price'
    }

    def __init__(self, order_item_number=None, ordered_item=None, order_item_quantity=None, price_currency=None, price=None):  # noqa: E501
        """OrderItem - a model defined in Swagger"""  # noqa: E501
        self._order_item_number = None
        self._ordered_item = None
        self._order_item_quantity = None
        self._price_currency = None
        self._price = None
        self.discriminator = None
        self.order_item_number = order_item_number
        self.ordered_item = ordered_item
        self.order_item_quantity = order_item_quantity
        self.price_currency = price_currency
        self.price = price

    @property
    def order_item_number(self):
        """Gets the order_item_number of this OrderItem.  # noqa: E501

        The number of order item  # noqa: E501

        :return: The order_item_number of this OrderItem.  # noqa: E501
        :rtype: str
        """
        return self._order_item_number

    @order_item_number.setter
    def order_item_number(self, order_item_number):
        """Sets the order_item_number of this OrderItem.

        The number of order item  # noqa: E501

        :param order_item_number: The order_item_number of this OrderItem.  # noqa: E501
        :type: str
        """
        if order_item_number is None:
            raise ValueError("Invalid value for `order_item_number`, must not be `None`")  # noqa: E501

        self._order_item_number = order_item_number

    @property
    def ordered_item(self):
        """Gets the ordered_item of this OrderItem.  # noqa: E501


        :return: The ordered_item of this OrderItem.  # noqa: E501
        :rtype: Thing
        """
        return self._ordered_item

    @ordered_item.setter
    def ordered_item(self, ordered_item):
        """Sets the ordered_item of this OrderItem.


        :param ordered_item: The ordered_item of this OrderItem.  # noqa: E501
        :type: Thing
        """
        if ordered_item is None:
            raise ValueError("Invalid value for `ordered_item`, must not be `None`")  # noqa: E501

        self._ordered_item = ordered_item

    @property
    def order_item_quantity(self):
        """Gets the order_item_quantity of this OrderItem.  # noqa: E501

        The quantity of order item  # noqa: E501

        :return: The order_item_quantity of this OrderItem.  # noqa: E501
        :rtype: int
        """
        return self._order_item_quantity

    @order_item_quantity.setter
    def order_item_quantity(self, order_item_quantity):
        """Sets the order_item_quantity of this OrderItem.

        The quantity of order item  # noqa: E501

        :param order_item_quantity: The order_item_quantity of this OrderItem.  # noqa: E501
        :type: int
        """
        if order_item_quantity is None:
            raise ValueError("Invalid value for `order_item_quantity`, must not be `None`")  # noqa: E501

        self._order_item_quantity = order_item_quantity

    @property
    def price_currency(self):
        """Gets the price_currency of this OrderItem.  # noqa: E501

        The symbol of fiat or digital token, such as USD, RMB, NEW,BTC,ETH.  # noqa: E501

        :return: The price_currency of this OrderItem.  # noqa: E501
        :rtype: str
        """
        return self._price_currency

    @price_currency.setter
    def price_currency(self, price_currency):
        """Sets the price_currency of this OrderItem.

        The symbol of fiat or digital token, such as USD, RMB, NEW,BTC,ETH.  # noqa: E501

        :param price_currency: The price_currency of this OrderItem.  # noqa: E501
        :type: str
        """
        if price_currency is None:
            raise ValueError("Invalid value for `price_currency`, must not be `None`")  # noqa: E501

        self._price_currency = price_currency

    @property
    def price(self):
        """Gets the price of this OrderItem.  # noqa: E501

        The amount of fiat or digital token, unit is the minimum unit of given fiat or digital token.  # noqa: E501

        :return: The price of this OrderItem.  # noqa: E501
        :rtype: str
        """
        return self._price

    @price.setter
    def price(self, price):
        """Sets the price of this OrderItem.

        The amount of fiat or digital token, unit is the minimum unit of given fiat or digital token.  # noqa: E501

        :param price: The price of this OrderItem.  # noqa: E501
        :type: str
        """
        if price is None:
            raise ValueError("Invalid value for `price`, must not be `None`")  # noqa: E501

        self._price = price

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(OrderItem, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrderItem):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    HEP REST API

    The REST API for HEP protocol  # noqa: E501

    OpenAPI spec version: v1
    Contact: xiawu@zeuux.org
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six


class RetrieveOracleResponse(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'oracle_id': 'str',
        'oracle_name': 'str',
        'public_key': 'str'
    }

    attribute_map = {
        'oracle_id': 'oracle_id',
        'oracle_name': 'oracle_name',
        'public_key': 'public_key'
    }

    def __init__(self, oracle_id=None, oracle_name=None, public_key=None):  # noqa: E501
        """RetrieveOracleResponse - a model defined in Swagger"""  # noqa: E501
        self._oracle_id = None
        self._oracle_name = None
        self._public_key = None
        self.discriminator = None
        self.oracle_id = oracle_id
        self.oracle_name = oracle_name
        self.public_key = public_key

    @property
    def oracle_id(self):
        """Gets the oracle_id of this RetrieveOracleResponse.  # noqa: E501

        The oracle id  # noqa: E501

        :return: The oracle_id of this RetrieveOracleResponse.  # noqa: E501
        :rtype: str
        """
        return self._oracle_id

    @oracle_id.setter
    def oracle_id(self, oracle_id):
        """Sets the oracle_id of this RetrieveOracleResponse.

        The oracle id  # noqa: E501

        :param oracle_id: The oracle_id of this RetrieveOracleResponse.  # noqa: E501
        :type: str
        """
        if oracle_id is None:
            raise ValueError("Invalid value for `oracle_id`, must not be `None`")  # noqa: E501

        self._oracle_id = oracle_id

    @property
    def oracle_name(self):
        """Gets the oracle_name of this RetrieveOracleResponse.  # noqa: E501

        The oracle name  # noqa: E501

        :return: The oracle_name of this RetrieveOracleResponse.  # noqa: E501
        :rtype: str
        """
        return self._oracle_name

    @oracle_name.setter
    def oracle_name(self, oracle_name):
        """Sets the oracle_name of this RetrieveOracleResponse.

        The oracle name  # noqa: E501

        :param oracle_name: The oracle_name of this RetrieveOracleResponse.  # noqa: E501
        :type: str
        """
        if oracle_name is None:
            raise ValueError("Invalid value for `oracle_name`, must not be `None`")  # noqa: E501

        self._oracle_name = oracle_name

    @property
    def public_key(self):
        """Gets the public_key of this RetrieveOracleResponse.  # noqa: E501

        The public key  # noqa: E501

        :return: The public_key of this RetrieveOracleResponse.  # noqa: E501
        :rtype: str
        """
        return self._public_key

    @public_key.setter
    def public_key(self, public_key):
        """Sets the public_key of this RetrieveOracleResponse.

        The public key  # noqa: E501

        :param public_key: The public_key of this RetrieveOracleResponse.  # noqa: E501
        :type: str
        """
        if public_key is None:
            raise ValueError("Invalid value for `public_key`, must not be `None`")  # noqa: E501

        self._public_key = public_key

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(RetrieveOracleResponse, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, RetrieveOracleResponse):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    HEP REST API

    The REST API for HEP protocol  # noqa: E501

    OpenAPI spec version: v1
    Contact: xiawu@zeuux.org
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six


class NewforceDailyStatsResponse(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'nf_new': 'str',
        'nf_cny': 'str',
        'nf_usd': 'str',
        'gnf': 'str'
    }

    attribute_map = {
        'nf_new': 'nf_new',
        'nf_cny': 'nf_cny',
        'nf_usd': 'nf_usd',
        'gnf': 'gnf'
    }

    def __init__(self, nf_new=None, nf_cny=None, nf_usd=None, gnf=None):  # noqa: E501
        """NewforceDailyStatsResponse - a model defined in Swagger"""  # noqa: E501
        self._nf_new = None
        self._nf_cny = None
        self._nf_usd = None
        self._gnf = None
        self.discriminator = None
        self.nf_new = nf_new
        self.nf_cny = nf_cny
        self.nf_usd = nf_usd
        self.gnf = gnf

    @property
    def nf_new(self):
        """Gets the nf_new of this NewforceDailyStatsResponse.  # noqa: E501

        The conversion rate between NewForce(NF) and NEW  # noqa: E501

        :return: The nf_new of this NewforceDailyStatsResponse.  # noqa: E501
        :rtype: str
        """
        return self._nf_new

    @nf_new.setter
    def nf_new(self, nf_new):
        """Sets the nf_new of this NewforceDailyStatsResponse.

        The conversion rate between NewForce(NF) and NEW  # noqa: E501

        :param nf_new: The nf_new of this NewforceDailyStatsResponse.  # noqa: E501
        :type: str
        """
        if nf_new is None:
            raise ValueError("Invalid value for `nf_new`, must not be `None`")  # noqa: E501

        self._nf_new = nf_new

    @property
    def nf_cny(self):
        """Gets the nf_cny of this NewforceDailyStatsResponse.  # noqa: E501

        The conversion rate between NewForce(NF) and CNY  # noqa: E501

        :return: The nf_cny of this NewforceDailyStatsResponse.  # noqa: E501
        :rtype: str
        """
        return self._nf_cny

    @nf_cny.setter
    def nf_cny(self, nf_cny):
        """Sets the nf_cny of this NewforceDailyStatsResponse.

        The conversion rate between NewForce(NF) and CNY  # noqa: E501

        :param nf_cny: The nf_cny of this NewforceDailyStatsResponse.  # noqa: E501
        :type: str
        """
        if nf_cny is None:
            raise ValueError("Invalid value for `nf_cny`, must not be `None`")  # noqa: E501

        self._nf_cny = nf_cny

    @property
    def nf_usd(self):
        """Gets the nf_usd of this NewforceDailyStatsResponse.  # noqa: E501

        The conversion rate between NewForce(NF) and USD  # noqa: E501

        :return: The nf_usd of this NewforceDailyStatsResponse.  # noqa: E501
        :rtype: str
        """
        return self._nf_usd

    @nf_usd.setter
    def nf_usd(self, nf_usd):
        """Sets the nf_usd of this NewforceDailyStatsResponse.

        The conversion rate between NewForce(NF) and USD  # noqa: E501

        :param nf_usd: The nf_usd of this NewforceDailyStatsResponse.  # noqa: E501
        :type: str
        """
        if nf_usd is None:
            raise ValueError("Invalid value for `nf_usd`, must not be `None`")  # noqa: E501

        self._nf_usd = nf_usd

    @property
    def gnf(self):
        """Gets the gnf of this NewforceDailyStatsResponse.  # noqa: E501

        The global NewForce  # noqa: E501

        :return: The gnf of this NewforceDailyStatsResponse.  # noqa: E501
        :rtype: str
        """
        return self._gnf

    @gnf.setter
    def gnf(self, gnf):
        """Sets the gnf of this NewforceDailyStatsResponse.

        The global NewForce  # noqa: E501

        :param gnf: The gnf of this NewforceDailyStatsResponse.  # noqa: E501
        :type: str
        """
        if gnf is None:
            raise ValueError("Invalid value for `gnf`, must not be `None`")  # noqa: E501

        self._gnf = gnf

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(NewforceDailyStatsResponse, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, NewforceDailyStatsResponse):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

# flake8: noqa

"""
    HEP REST API

    The REST API for HEP protocol  # noqa: E501

    OpenAPI spec version: v1
    Contact: xiawu@zeuux.org
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

# import apis into sdk package
from hep_rest_api.api.rest_api import RestApi
# import ApiClient
from hep_rest_api.api_client import ApiClient
from hep_rest_api.configuration import Configuration
# import models into sdk package
from hep_rest_api.models.auth_cache_request import AuthCacheRequest
from hep_rest_api.models.auth_cache_response import AuthCacheResponse
from hep_rest_api.models.cancel_proof_request import CancelProofRequest
from hep_rest_api.models.cancel_proof_response import CancelProofResponse
from hep_rest_api.models.create_auth_cache_response import CreateAuthCacheResponse
from hep_rest_api.models.create_pay_cache_response import CreatePayCacheResponse
from hep_rest_api.models.create_proof_request import CreateProofRequest
from hep_rest_api.models.create_proof_response import CreateProofResponse
from hep_rest_api.models.dapp import Dapp
from hep_rest_api.models.newchain_transaction_response import NewchainTransactionResponse
from hep_rest_api.models.newforce_daily_stats_response import NewforceDailyStatsResponse
from hep_rest_api.models.newid_response import NewidResponse
from hep_rest_api.models.order import Order
from hep_rest_api.models.order_item import OrderItem
from hep_rest_api.models.pay_cache_request import PayCacheRequest
from hep_rest_api.models.pay_cache_response import PayCacheResponse
from hep_rest_api.models.proof import Proof
from hep_rest_api.models.proof_response import ProofResponse
from hep_rest_api.models.receipt_field import ReceiptField
from hep_rest_api.models.retrieve_oracle_response import RetrieveOracleResponse
from hep_rest_api.models.retrieve_proof_receipts_request import RetrieveProofReceiptsRequest
from hep_rest_api.models.retrieve_proof_receipts_response import RetrieveProofReceiptsResponse
from hep_rest_api.models.thing import Thing



"""The implementation of HEP Proof protocol

"""

__copyright__ = """ Copyright (c) 2019 Newton Foundation. All rights reserved."""
__version__ = '1.0'
__author__ = 'xiawu@zeuux.org'

import logging

from hep_rest_api import models
from hep_rest_api.scenarios import BaseHelper

logger = logging.getLogger(__name__)


class Order(object):
    """docstring for Order"""

    def __init__(self,
                 order_number,  # The identifier of the order.
                 description,  # The description of the order.
                 total_price,  # The total price of the order.
                 price_currency,  # The price currency  of the order.
                 seller,  # The order seller's NEWID.
                 customer,  # The order customer's NEWID.
                 broker='',  # The order broker's NEWID.
                 chain_txid='',  # The transaction id of the order.
                 ):
        self.order_number = order_number
        self.description = description
        self.total_price = total_price
        self.price_currency = price_currency
        self.seller = seller
        self.customer = customer
        self.broker = broker
        self.chain_txid = chain_txid
        self.order_items = []

    def add_order_item(self,
                       order_item_number,  # The identifier of the order item.
                       order_item_quantity,
                       # The number of the item ordered. If the property is not set, assume the quantity is one.
                       price,  # The order item's price
                       price_currency,  # The symbol of fiat or digital token, such as USD, CNY, NEW,BTC,ETH.
                       thing_name,  # The thing name such as product or service name.
                       thing_id,  # The thing id such as sku, ISBN.
                       thing_type='product',  # The thing type such as product or service.
                       ):
        self.order_items.append({
            'order_item_number': order_item_number,
            'order_item_quantity': order_item_quantity,
            'price': price,
            'price_currency': price_currency,
            'ordered_item': {
                'name': thing_name,
                'thing_id': thing_id,
                'thing_type': thing_type,
            },
        })

    def to_dict(self):
        return {
            'order_number': self.order_number,
            'description': self.description,
            'total_price': self.total_price,
            'price_currency': self.price_currency,
            'seller': self.seller,
            'customer': self.customer,
            'broker': self.broker,
            'chain_txid': self.chain_txid,
            'order_items': self.order_items
        }


class OrderProof(object):
    def __init__(self,
                 total_price,
                 # The amount of fiat or digital token, unit is the minimum unit of given fiat or digital token.
                 price_currency,  # The symbol of fiat or digital token, such as USD, RMB, NEW,BTC,ETH.
                 submitter,  # The submitter's NewID
                 proof_type='order',  # The proof type which value is "order".
                 ):
        self.price_currency = price_currency
        self.total_price = total_price
        self.submitter = submitter
        self.proof_type = proof_type
        self.orders = []

    def add_order(self,
                  order,  # The order item.
                  ):
        self.orders.append(order)

    def to_dict(self):
        return {
            'price_currency': self.price_currency,
            'total_price': self.total_price,
            'submitter': self.submitter,
            'proof_type': self.proof_type,
            'orders': self.orders
        }


class ProofHelper(BaseHelper):
    def generate_proof_request(self,
                               content,
                               uuid=None):
        """Generate the payment request
        
        :param str dapp_id: The DApp ID
        :param dict content: The content. it can be generated by OrderProof utility.
        :param str action: The request action name
        :param str uuid: The request uuid for desktop-browser. optional.
        :rtype: dict
        :return: The request information including signature
        """
        if not uuid:
            raise AttributeError(
                "uuid can not be None"
            )
        if not content:
            raise AttributeError(
                "content can not be None"
            )
        data = {
            'action': self.action_auth_proof,
            'uuid': uuid,
            'content': content,
            'dapp_id': self.dapp_id
        }
        sign_data = self.generate_sign_data(data)
        hmac_data = self.sign_hmac(sign_data)
        final_data = self.sign_secp256r1(hmac_data)
        # start request
        proofs = models.CreateProofRequest(**final_data)
        auth_response = self.api_client.rest_proofs_create(proofs, self.api_version)
        return auth_response

    def generate_qrcode_string(self, proof_hash):
        """Generate the hep-based scheme string string for QRCode 
        
        :param dict proof_hash: The proof request hash
        :rtype: str
        :return: The QRcode string
        """
        protocol = self.base_parameters.get('protocol')
        action = self.action_auth_proof
        qrcode_str = "%s://%s/?action=%s&proof_hash=%s" % (protocol.lower(), self.dapp_id, action, proof_hash)
        return qrcode_str

    def validate_proof_callback(self, data):
        """Validate the response of proof callback sent by end users such as NewPay users 
        
        :param dict data: The callback data
        :rtype: bool
        :return: True if valid data, otherwise False
        """
        return self.validate_r1_data(data)

    def get_status_of_proofs(self, proof_hashes):
        """Get the status of given proof_hashes.eg: SUBMIT, CONFIRM, PROCESSING, PART_CANCELED, CANCELED

        :param proof_hashes: list
        :rtype: RetrieveProofReceiptsResponse
        :return: The list of proofs status
        """
        params = {
            'proof_hashes': proof_hashes
        }
        sign_data = self.generate_sign_data(params)
        hmac_data = self.sign_hmac(sign_data)
        proof_request = models.RetrieveProofReceiptsRequest(**hmac_data)
        response = self.api_client.rest_proofs_receipts_create(body=proof_request, api_version=self.api_version)
        return response



"""The implementation of HEP Payment protocol

"""

__copyright__ = """ Copyright (c) 2019 Newton Foundation. All rights reserved."""
__version__ = '1.0'
__author__ = 'xiawu@zeuux.org'

import datetime
import logging
from hep_rest_api.scenarios import BaseHelper
from hep_rest_api import models

logger = logging.getLogger(__name__)


class PayHelper(BaseHelper):
    def generate_pay_request(self,
                             order_number,
                             price_currency,
                             total_price,
                             description,
                             seller,
                             customer,
                             broker='',
                             expired=300,
                             uuid=None):
        """Generate the payment request
        
        :param str dapp_id: The DApp ID
        :param str order_number: The order number
        :param str price_currency: The symbol of fiat or digital token, such as USD, CNY, NEW,BTC,ETH.
        :param str total_price: The amount of fiat or digital token, unit is the minimum unit of given fiat or digital token.
        :param str description: The order description
        :param str seller: The seller's NewID
        :param str customer: The customer's NewID
        :param str broker: The broker's NewID. optional.
        :param int expired: The expired time of request
        :param str uuid: The request uuid for desktop-browser. optional.
        :rtype: dict
        :return: The request information including signature
        """
        if not uuid:
            raise AttributeError(
                "uuid can not be None"
            )
        if not order_number:
            raise AttributeError(
                "order number can not be None"
            )
        data = {
            'action': self.action_auth_pay,
            'order_number': order_number,
            'expired': int(datetime.datetime.now().timestamp()) + expired,
            'price_currency': price_currency,
            'total_price': total_price,
            'description': description,
            'seller': seller,
            'customer': customer,
            'broker': broker,
            'uuid': uuid,
            'dapp_id': self.dapp_id
        }
        sign_data = self.generate_sign_data(data)
        hmac_data = self.sign_hmac(sign_data)
        final_data = self.sign_secp256r1(hmac_data)
        # start request
        auth_cache = models.PayCacheRequest(**final_data)
        auth_response = self.api_client.rest_newnet_caches_pay_create(auth_cache, self.api_version)
        return auth_response

    def generate_qrcode_string(self, pay_hash):
        """Generate the hep-based scheme string string for QRCode 
        
        :param str pay_hash: The pay request hash
        :rtype: str
        :return: The QRcode string
        """
        protocol = self.base_parameters.get('protocol')
        action = self.action_auth_pay
        qrcode_str = "%s://%s/?action=%s&pay_hash=%s" % (protocol.lower(), self.dapp_id, action, pay_hash)
        return qrcode_str

    def validate_pay_callback(self, data):
        """Validate the response of pay callback sent by end users such as NewPay users 
        
        :param dict data: The callback data
        :rtype: bool
        :return: True if valid data, otherwise False
        """
        return self.validate_r1_data(data)

    def get_confirmed_transaction(self, transaction_id):
        """ Validate transaction by transaction id. and you can get the order_number, block height and etc.

        :param transaction_id: txid
        :rtype: dict
        :return: dict about transaction
        """
        params = {}
        sign_data = self.generate_sign_data(params)
        hmac_data = self.sign_hmac(sign_data)
        hmac_data['txid'] = transaction_id
        hmac_data['api_version'] = self.api_version
        response = self.api_client.rest_newchain_tx_read(**hmac_data)
        return response






# coding: utf-8

"""
    HEP REST API

    The REST API for HEP protocol  # noqa: E501

    OpenAPI spec version: v1
    Contact: xiawu@zeuux.org
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six


class NewidResponse(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'newid': 'str',
        'public_key': 'str',
        'main_wallet_address': 'str'
    }

    attribute_map = {
        'newid': 'newid',
        'public_key': 'public_key',
        'main_wallet_address': 'main_wallet_address'
    }

    def __init__(self, newid=None, public_key=None, main_wallet_address=None):  # noqa: E501
        """NewidResponse - a model defined in Swagger"""  # noqa: E501
        self._newid = None
        self._public_key = None
        self._main_wallet_address = None
        self.discriminator = None
        self.newid = newid
        self.public_key = public_key
        self.main_wallet_address = main_wallet_address

    @property
    def newid(self):
        """Gets the newid of this NewidResponse.  # noqa: E501

        The NewID  # noqa: E501

        :return: The newid of this NewidResponse.  # noqa: E501
        :rtype: str
        """
        return self._newid

    @newid.setter
    def newid(self, newid):
        """Sets the newid of this NewidResponse.

        The NewID  # noqa: E501

        :param newid: The newid of this NewidResponse.  # noqa: E501
        :type: str
        """
        if newid is None:
            raise ValueError("Invalid value for `newid`, must not be `None`")  # noqa: E501

        self._newid = newid

    @property
    def public_key(self):
        """Gets the public_key of this NewidResponse.  # noqa: E501

        The hex string of NewID's public key  # noqa: E501

        :return: The public_key of this NewidResponse.  # noqa: E501
        :rtype: str
        """
        return self._public_key

    @public_key.setter
    def public_key(self, public_key):
        """Sets the public_key of this NewidResponse.

        The hex string of NewID's public key  # noqa: E501

        :param public_key: The public_key of this NewidResponse.  # noqa: E501
        :type: str
        """
        if public_key is None:
            raise ValueError("Invalid value for `public_key`, must not be `None`")  # noqa: E501

        self._public_key = public_key

    @property
    def main_wallet_address(self):
        """Gets the main_wallet_address of this NewidResponse.  # noqa: E501

        The main wallet address of NewChain  # noqa: E501

        :return: The main_wallet_address of this NewidResponse.  # noqa: E501
        :rtype: str
        """
        return self._main_wallet_address

    @main_wallet_address.setter
    def main_wallet_address(self, main_wallet_address):
        """Sets the main_wallet_address of this NewidResponse.

        The main wallet address of NewChain  # noqa: E501

        :param main_wallet_address: The main_wallet_address of this NewidResponse.  # noqa: E501
        :type: str
        """
        if main_wallet_address is None:
            raise ValueError("Invalid value for `main_wallet_address`, must not be `None`")  # noqa: E501

        self._main_wallet_address = main_wallet_address

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(NewidResponse, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, NewidResponse):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    HEP REST API

    The REST API for HEP protocol  # noqa: E501

    OpenAPI spec version: v1
    Contact: xiawu@zeuux.org
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six


class RetrieveProofReceiptsRequest(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'dapp_key': 'str',
        'protocol': 'str',
        'version': 'str',
        'ts': 'int',
        'nonce': 'str',
        'os': 'str',
        'language': 'str',
        'dapp_signature_method': 'str',
        'dapp_signature': 'str',
        'proof_hashes': 'list[str]'
    }

    attribute_map = {
        'dapp_key': 'dapp_key',
        'protocol': 'protocol',
        'version': 'version',
        'ts': 'ts',
        'nonce': 'nonce',
        'os': 'os',
        'language': 'language',
        'dapp_signature_method': 'dapp_signature_method',
        'dapp_signature': 'dapp_signature',
        'proof_hashes': 'proof_hashes'
    }

    def __init__(self, dapp_key=None, protocol=None, version=None, ts=None, nonce=None, os=None, language=None, dapp_signature_method=None, dapp_signature=None, proof_hashes=None):  # noqa: E501
        """RetrieveProofReceiptsRequest - a model defined in Swagger"""  # noqa: E501
        self._dapp_key = None
        self._protocol = None
        self._version = None
        self._ts = None
        self._nonce = None
        self._os = None
        self._language = None
        self._dapp_signature_method = None
        self._dapp_signature = None
        self._proof_hashes = None
        self.discriminator = None
        self.dapp_key = dapp_key
        self.protocol = protocol
        self.version = version
        self.ts = ts
        self.nonce = nonce
        self.os = os
        self.language = language
        self.dapp_signature_method = dapp_signature_method
        self.dapp_signature = dapp_signature
        self.proof_hashes = proof_hashes

    @property
    def dapp_key(self):
        """Gets the dapp_key of this RetrieveProofReceiptsRequest.  # noqa: E501

        The decentralized application access key  # noqa: E501

        :return: The dapp_key of this RetrieveProofReceiptsRequest.  # noqa: E501
        :rtype: str
        """
        return self._dapp_key

    @dapp_key.setter
    def dapp_key(self, dapp_key):
        """Sets the dapp_key of this RetrieveProofReceiptsRequest.

        The decentralized application access key  # noqa: E501

        :param dapp_key: The dapp_key of this RetrieveProofReceiptsRequest.  # noqa: E501
        :type: str
        """
        if dapp_key is None:
            raise ValueError("Invalid value for `dapp_key`, must not be `None`")  # noqa: E501

        self._dapp_key = dapp_key

    @property
    def protocol(self):
        """Gets the protocol of this RetrieveProofReceiptsRequest.  # noqa: E501

        The protocol name. default is 'HEP'.  # noqa: E501

        :return: The protocol of this RetrieveProofReceiptsRequest.  # noqa: E501
        :rtype: str
        """
        return self._protocol

    @protocol.setter
    def protocol(self, protocol):
        """Sets the protocol of this RetrieveProofReceiptsRequest.

        The protocol name. default is 'HEP'.  # noqa: E501

        :param protocol: The protocol of this RetrieveProofReceiptsRequest.  # noqa: E501
        :type: str
        """
        if protocol is None:
            raise ValueError("Invalid value for `protocol`, must not be `None`")  # noqa: E501

        self._protocol = protocol

    @property
    def version(self):
        """Gets the version of this RetrieveProofReceiptsRequest.  # noqa: E501

        The protocol version such as '1.0'  # noqa: E501

        :return: The version of this RetrieveProofReceiptsRequest.  # noqa: E501
        :rtype: str
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this RetrieveProofReceiptsRequest.

        The protocol version such as '1.0'  # noqa: E501

        :param version: The version of this RetrieveProofReceiptsRequest.  # noqa: E501
        :type: str
        """
        if version is None:
            raise ValueError("Invalid value for `version`, must not be `None`")  # noqa: E501

        self._version = version

    @property
    def ts(self):
        """Gets the ts of this RetrieveProofReceiptsRequest.  # noqa: E501

        The current timestamp  # noqa: E501

        :return: The ts of this RetrieveProofReceiptsRequest.  # noqa: E501
        :rtype: int
        """
        return self._ts

    @ts.setter
    def ts(self, ts):
        """Sets the ts of this RetrieveProofReceiptsRequest.

        The current timestamp  # noqa: E501

        :param ts: The ts of this RetrieveProofReceiptsRequest.  # noqa: E501
        :type: int
        """
        if ts is None:
            raise ValueError("Invalid value for `ts`, must not be `None`")  # noqa: E501

        self._ts = ts

    @property
    def nonce(self):
        """Gets the nonce of this RetrieveProofReceiptsRequest.  # noqa: E501

        The random string or auto-increment sequence  # noqa: E501

        :return: The nonce of this RetrieveProofReceiptsRequest.  # noqa: E501
        :rtype: str
        """
        return self._nonce

    @nonce.setter
    def nonce(self, nonce):
        """Sets the nonce of this RetrieveProofReceiptsRequest.

        The random string or auto-increment sequence  # noqa: E501

        :param nonce: The nonce of this RetrieveProofReceiptsRequest.  # noqa: E501
        :type: str
        """
        if nonce is None:
            raise ValueError("Invalid value for `nonce`, must not be `None`")  # noqa: E501

        self._nonce = nonce

    @property
    def os(self):
        """Gets the os of this RetrieveProofReceiptsRequest.  # noqa: E501

        The operating system of client such as ios, android, dweb,etc.  # noqa: E501

        :return: The os of this RetrieveProofReceiptsRequest.  # noqa: E501
        :rtype: str
        """
        return self._os

    @os.setter
    def os(self, os):
        """Sets the os of this RetrieveProofReceiptsRequest.

        The operating system of client such as ios, android, dweb,etc.  # noqa: E501

        :param os: The os of this RetrieveProofReceiptsRequest.  # noqa: E501
        :type: str
        """
        if os is None:
            raise ValueError("Invalid value for `os`, must not be `None`")  # noqa: E501

        self._os = os

    @property
    def language(self):
        """Gets the language of this RetrieveProofReceiptsRequest.  # noqa: E501

        The i18n language code such as zh, en, etc.  # noqa: E501

        :return: The language of this RetrieveProofReceiptsRequest.  # noqa: E501
        :rtype: str
        """
        return self._language

    @language.setter
    def language(self, language):
        """Sets the language of this RetrieveProofReceiptsRequest.

        The i18n language code such as zh, en, etc.  # noqa: E501

        :param language: The language of this RetrieveProofReceiptsRequest.  # noqa: E501
        :type: str
        """
        if language is None:
            raise ValueError("Invalid value for `language`, must not be `None`")  # noqa: E501

        self._language = language

    @property
    def dapp_signature_method(self):
        """Gets the dapp_signature_method of this RetrieveProofReceiptsRequest.  # noqa: E501

        The signature method used by dapp.  # noqa: E501

        :return: The dapp_signature_method of this RetrieveProofReceiptsRequest.  # noqa: E501
        :rtype: str
        """
        return self._dapp_signature_method

    @dapp_signature_method.setter
    def dapp_signature_method(self, dapp_signature_method):
        """Sets the dapp_signature_method of this RetrieveProofReceiptsRequest.

        The signature method used by dapp.  # noqa: E501

        :param dapp_signature_method: The dapp_signature_method of this RetrieveProofReceiptsRequest.  # noqa: E501
        :type: str
        """
        if dapp_signature_method is None:
            raise ValueError("Invalid value for `dapp_signature_method`, must not be `None`")  # noqa: E501

        self._dapp_signature_method = dapp_signature_method

    @property
    def dapp_signature(self):
        """Gets the dapp_signature of this RetrieveProofReceiptsRequest.  # noqa: E501

        The signature generated by dapp.  # noqa: E501

        :return: The dapp_signature of this RetrieveProofReceiptsRequest.  # noqa: E501
        :rtype: str
        """
        return self._dapp_signature

    @dapp_signature.setter
    def dapp_signature(self, dapp_signature):
        """Sets the dapp_signature of this RetrieveProofReceiptsRequest.

        The signature generated by dapp.  # noqa: E501

        :param dapp_signature: The dapp_signature of this RetrieveProofReceiptsRequest.  # noqa: E501
        :type: str
        """
        if dapp_signature is None:
            raise ValueError("Invalid value for `dapp_signature`, must not be `None`")  # noqa: E501

        self._dapp_signature = dapp_signature

    @property
    def proof_hashes(self):
        """Gets the proof_hashes of this RetrieveProofReceiptsRequest.  # noqa: E501

        The proof hashes  # noqa: E501

        :return: The proof_hashes of this RetrieveProofReceiptsRequest.  # noqa: E501
        :rtype: list[str]
        """
        return self._proof_hashes

    @proof_hashes.setter
    def proof_hashes(self, proof_hashes):
        """Sets the proof_hashes of this RetrieveProofReceiptsRequest.

        The proof hashes  # noqa: E501

        :param proof_hashes: The proof_hashes of this RetrieveProofReceiptsRequest.  # noqa: E501
        :type: list[str]
        """
        if proof_hashes is None:
            raise ValueError("Invalid value for `proof_hashes`, must not be `None`")  # noqa: E501

        self._proof_hashes = proof_hashes

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(RetrieveProofReceiptsRequest, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, RetrieveProofReceiptsRequest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    HEP REST API

    The REST API for HEP protocol  # noqa: E501

    OpenAPI spec version: v1
    Contact: xiawu@zeuux.org
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six


class Dapp(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'dapp_id': 'str',
        'dapp_name': 'str',
        'icon': 'str',
        'dapp_public_key': 'str',
        'package_name': 'str',
        'bundle_id': 'str',
        'schema': 'str',
        'website': 'str',
        'deposit_contract_address': 'str',
        'dapp_type_ids': 'list[int]',
        'dapp_category_id': 'int',
        'auth_login_callback': 'str',
        'pay_order_callback': 'str',
        'proof_submit_callback': 'str'
    }

    attribute_map = {
        'dapp_id': 'dapp_id',
        'dapp_name': 'dapp_name',
        'icon': 'icon',
        'dapp_public_key': 'dapp_public_key',
        'package_name': 'package_name',
        'bundle_id': 'bundle_id',
        'schema': 'schema',
        'website': 'website',
        'deposit_contract_address': 'deposit_contract_address',
        'dapp_type_ids': 'dapp_type_ids',
        'dapp_category_id': 'dapp_category_id',
        'auth_login_callback': 'auth_login_callback',
        'pay_order_callback': 'pay_order_callback',
        'proof_submit_callback': 'proof_submit_callback'
    }

    def __init__(self, dapp_id=None, dapp_name=None, icon=None, dapp_public_key=None, package_name=None, bundle_id=None, schema=None, website=None, deposit_contract_address=None, dapp_type_ids=None, dapp_category_id=None, auth_login_callback=None, pay_order_callback=None, proof_submit_callback=None):  # noqa: E501
        """Dapp - a model defined in Swagger"""  # noqa: E501
        self._dapp_id = None
        self._dapp_name = None
        self._icon = None
        self._dapp_public_key = None
        self._package_name = None
        self._bundle_id = None
        self._schema = None
        self._website = None
        self._deposit_contract_address = None
        self._dapp_type_ids = None
        self._dapp_category_id = None
        self._auth_login_callback = None
        self._pay_order_callback = None
        self._proof_submit_callback = None
        self.discriminator = None
        self.dapp_id = dapp_id
        self.dapp_name = dapp_name
        self.icon = icon
        self.dapp_public_key = dapp_public_key
        self.package_name = package_name
        self.bundle_id = bundle_id
        self.schema = schema
        self.website = website
        self.deposit_contract_address = deposit_contract_address
        self.dapp_type_ids = dapp_type_ids
        self.dapp_category_id = dapp_category_id
        self.auth_login_callback = auth_login_callback
        self.pay_order_callback = pay_order_callback
        self.proof_submit_callback = proof_submit_callback

    @property
    def dapp_id(self):
        """Gets the dapp_id of this Dapp.  # noqa: E501

        The decentralized application ID  # noqa: E501

        :return: The dapp_id of this Dapp.  # noqa: E501
        :rtype: str
        """
        return self._dapp_id

    @dapp_id.setter
    def dapp_id(self, dapp_id):
        """Sets the dapp_id of this Dapp.

        The decentralized application ID  # noqa: E501

        :param dapp_id: The dapp_id of this Dapp.  # noqa: E501
        :type: str
        """
        if dapp_id is None:
            raise ValueError("Invalid value for `dapp_id`, must not be `None`")  # noqa: E501

        self._dapp_id = dapp_id

    @property
    def dapp_name(self):
        """Gets the dapp_name of this Dapp.  # noqa: E501

        The decentralized application name  # noqa: E501

        :return: The dapp_name of this Dapp.  # noqa: E501
        :rtype: str
        """
        return self._dapp_name

    @dapp_name.setter
    def dapp_name(self, dapp_name):
        """Sets the dapp_name of this Dapp.

        The decentralized application name  # noqa: E501

        :param dapp_name: The dapp_name of this Dapp.  # noqa: E501
        :type: str
        """
        if dapp_name is None:
            raise ValueError("Invalid value for `dapp_name`, must not be `None`")  # noqa: E501

        self._dapp_name = dapp_name

    @property
    def icon(self):
        """Gets the icon of this Dapp.  # noqa: E501

        The icon of application  # noqa: E501

        :return: The icon of this Dapp.  # noqa: E501
        :rtype: str
        """
        return self._icon

    @icon.setter
    def icon(self, icon):
        """Sets the icon of this Dapp.

        The icon of application  # noqa: E501

        :param icon: The icon of this Dapp.  # noqa: E501
        :type: str
        """
        if icon is None:
            raise ValueError("Invalid value for `icon`, must not be `None`")  # noqa: E501

        self._icon = icon

    @property
    def dapp_public_key(self):
        """Gets the dapp_public_key of this Dapp.  # noqa: E501

        The public key of DApp  # noqa: E501

        :return: The dapp_public_key of this Dapp.  # noqa: E501
        :rtype: str
        """
        return self._dapp_public_key

    @dapp_public_key.setter
    def dapp_public_key(self, dapp_public_key):
        """Sets the dapp_public_key of this Dapp.

        The public key of DApp  # noqa: E501

        :param dapp_public_key: The dapp_public_key of this Dapp.  # noqa: E501
        :type: str
        """
        if dapp_public_key is None:
            raise ValueError("Invalid value for `dapp_public_key`, must not be `None`")  # noqa: E501

        self._dapp_public_key = dapp_public_key

    @property
    def package_name(self):
        """Gets the package_name of this Dapp.  # noqa: E501

        The package name such as com.demo.dev.android  # noqa: E501

        :return: The package_name of this Dapp.  # noqa: E501
        :rtype: str
        """
        return self._package_name

    @package_name.setter
    def package_name(self, package_name):
        """Sets the package_name of this Dapp.

        The package name such as com.demo.dev.android  # noqa: E501

        :param package_name: The package_name of this Dapp.  # noqa: E501
        :type: str
        """
        if package_name is None:
            raise ValueError("Invalid value for `package_name`, must not be `None`")  # noqa: E501

        self._package_name = package_name

    @property
    def bundle_id(self):
        """Gets the bundle_id of this Dapp.  # noqa: E501

        The bundle id such as com.demo.dev.ios for iOS platform  # noqa: E501

        :return: The bundle_id of this Dapp.  # noqa: E501
        :rtype: str
        """
        return self._bundle_id

    @bundle_id.setter
    def bundle_id(self, bundle_id):
        """Sets the bundle_id of this Dapp.

        The bundle id such as com.demo.dev.ios for iOS platform  # noqa: E501

        :param bundle_id: The bundle_id of this Dapp.  # noqa: E501
        :type: str
        """
        if bundle_id is None:
            raise ValueError("Invalid value for `bundle_id`, must not be `None`")  # noqa: E501

        self._bundle_id = bundle_id

    @property
    def schema(self):
        """Gets the schema of this Dapp.  # noqa: E501

        The routing schema  # noqa: E501

        :return: The schema of this Dapp.  # noqa: E501
        :rtype: str
        """
        return self._schema

    @schema.setter
    def schema(self, schema):
        """Sets the schema of this Dapp.

        The routing schema  # noqa: E501

        :param schema: The schema of this Dapp.  # noqa: E501
        :type: str
        """
        if schema is None:
            raise ValueError("Invalid value for `schema`, must not be `None`")  # noqa: E501

        self._schema = schema

    @property
    def website(self):
        """Gets the website of this Dapp.  # noqa: E501

        The dapp website link  # noqa: E501

        :return: The website of this Dapp.  # noqa: E501
        :rtype: str
        """
        return self._website

    @website.setter
    def website(self, website):
        """Sets the website of this Dapp.

        The dapp website link  # noqa: E501

        :param website: The website of this Dapp.  # noqa: E501
        :type: str
        """
        if website is None:
            raise ValueError("Invalid value for `website`, must not be `None`")  # noqa: E501

        self._website = website

    @property
    def deposit_contract_address(self):
        """Gets the deposit_contract_address of this Dapp.  # noqa: E501

        The deposit contract Address, the example is NEW182....  # noqa: E501

        :return: The deposit_contract_address of this Dapp.  # noqa: E501
        :rtype: str
        """
        return self._deposit_contract_address

    @deposit_contract_address.setter
    def deposit_contract_address(self, deposit_contract_address):
        """Sets the deposit_contract_address of this Dapp.

        The deposit contract Address, the example is NEW182....  # noqa: E501

        :param deposit_contract_address: The deposit_contract_address of this Dapp.  # noqa: E501
        :type: str
        """
        if deposit_contract_address is None:
            raise ValueError("Invalid value for `deposit_contract_address`, must not be `None`")  # noqa: E501

        self._deposit_contract_address = deposit_contract_address

    @property
    def dapp_type_ids(self):
        """Gets the dapp_type_ids of this Dapp.  # noqa: E501

        The support dapp type list.  # noqa: E501

        :return: The dapp_type_ids of this Dapp.  # noqa: E501
        :rtype: list[int]
        """
        return self._dapp_type_ids

    @dapp_type_ids.setter
    def dapp_type_ids(self, dapp_type_ids):
        """Sets the dapp_type_ids of this Dapp.

        The support dapp type list.  # noqa: E501

        :param dapp_type_ids: The dapp_type_ids of this Dapp.  # noqa: E501
        :type: list[int]
        """
        if dapp_type_ids is None:
            raise ValueError("Invalid value for `dapp_type_ids`, must not be `None`")  # noqa: E501

        self._dapp_type_ids = dapp_type_ids

    @property
    def dapp_category_id(self):
        """Gets the dapp_category_id of this Dapp.  # noqa: E501

        The dapp category ID.  # noqa: E501

        :return: The dapp_category_id of this Dapp.  # noqa: E501
        :rtype: int
        """
        return self._dapp_category_id

    @dapp_category_id.setter
    def dapp_category_id(self, dapp_category_id):
        """Sets the dapp_category_id of this Dapp.

        The dapp category ID.  # noqa: E501

        :param dapp_category_id: The dapp_category_id of this Dapp.  # noqa: E501
        :type: int
        """
        if dapp_category_id is None:
            raise ValueError("Invalid value for `dapp_category_id`, must not be `None`")  # noqa: E501

        self._dapp_category_id = dapp_category_id

    @property
    def auth_login_callback(self):
        """Gets the auth_login_callback of this Dapp.  # noqa: E501

        For Mobile Native DApp, it is redirect schema; For website DApp, it is callback url; For  NewDApp, it is HEP-based url.  # noqa: E501

        :return: The auth_login_callback of this Dapp.  # noqa: E501
        :rtype: str
        """
        return self._auth_login_callback

    @auth_login_callback.setter
    def auth_login_callback(self, auth_login_callback):
        """Sets the auth_login_callback of this Dapp.

        For Mobile Native DApp, it is redirect schema; For website DApp, it is callback url; For  NewDApp, it is HEP-based url.  # noqa: E501

        :param auth_login_callback: The auth_login_callback of this Dapp.  # noqa: E501
        :type: str
        """
        if auth_login_callback is None:
            raise ValueError("Invalid value for `auth_login_callback`, must not be `None`")  # noqa: E501

        self._auth_login_callback = auth_login_callback

    @property
    def pay_order_callback(self):
        """Gets the pay_order_callback of this Dapp.  # noqa: E501

        For Mobile Native DApp, it is redirect schema; For website DApp, it is callback url; For  NewDApp, it is HEP-based url.  # noqa: E501

        :return: The pay_order_callback of this Dapp.  # noqa: E501
        :rtype: str
        """
        return self._pay_order_callback

    @pay_order_callback.setter
    def pay_order_callback(self, pay_order_callback):
        """Sets the pay_order_callback of this Dapp.

        For Mobile Native DApp, it is redirect schema; For website DApp, it is callback url; For  NewDApp, it is HEP-based url.  # noqa: E501

        :param pay_order_callback: The pay_order_callback of this Dapp.  # noqa: E501
        :type: str
        """
        if pay_order_callback is None:
            raise ValueError("Invalid value for `pay_order_callback`, must not be `None`")  # noqa: E501

        self._pay_order_callback = pay_order_callback

    @property
    def proof_submit_callback(self):
        """Gets the proof_submit_callback of this Dapp.  # noqa: E501

        For Mobile Native DApp, it is redirect schema; For website DApp, it is callback url; For  NewDApp, it is HEP-based url.  # noqa: E501

        :return: The proof_submit_callback of this Dapp.  # noqa: E501
        :rtype: str
        """
        return self._proof_submit_callback

    @proof_submit_callback.setter
    def proof_submit_callback(self, proof_submit_callback):
        """Sets the proof_submit_callback of this Dapp.

        For Mobile Native DApp, it is redirect schema; For website DApp, it is callback url; For  NewDApp, it is HEP-based url.  # noqa: E501

        :param proof_submit_callback: The proof_submit_callback of this Dapp.  # noqa: E501
        :type: str
        """
        if proof_submit_callback is None:
            raise ValueError("Invalid value for `proof_submit_callback`, must not be `None`")  # noqa: E501

        self._proof_submit_callback = proof_submit_callback

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Dapp, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Dapp):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



"""The abstract layer of HEP scenarios

"""

__copyright__ = """ Copyright (c) 2019 Newton Foundation. All rights reserved."""
__version__ = '1.0'
__author__ = 'xiawu@zeuux.org'

import logging
import copy
import datetime
import uuid
from hep_rest_api import utils

logger = logging.getLogger(__name__)


class BaseHelper(object):
    def __init__(self, api_client, base_parameters, dapp_id, dapp_secret, key_path, api_version='1', chain_id=1012):
        """Initialize the basic parameters

        :param hep_rest_api.RestApi api_client: The api client instance
        :param dict base_parameters: The base parameters for HMAC authentication
        :param str key_path: The private key path for dapp owner's signature
        :param dapp_id: The dapp id for dapp owner's
        """
        # verify the required parameter 'dapp_key' is set
        if ('dapp_key' not in base_parameters or
                base_parameters['dapp_key'] is None):
            raise ValueError("Missing the required parameter `dapp_key` when calling `BaseHelper`")  # noqa: E501
        # verify the required parameter 'protocol' is set
        if ('protocol' not in base_parameters or
                base_parameters['protocol'] is None):
            raise ValueError("Missing the required parameter `protocol` when calling `BaseHelper`")  # noqa: E501
        # verify the required parameter 'version' is set
        if ('version' not in base_parameters or
                base_parameters['version'] is None):
            raise ValueError("Missing the required parameter `version` when calling `BaseHelper`")  # noqa: E501
        if ('os' not in base_parameters or
                base_parameters['os'] is None):
            raise ValueError("Missing the required parameter `os` when calling `BaseHelper`")  # noqa: E501
        # verify the required parameter 'language' is set
        if ('language' not in base_parameters or
                base_parameters['language'] is None):
            raise ValueError("Missing the required parameter `language` when calling `BaseHelper`")  # noqa: E501
        self.api_client = api_client
        self.base_parameters = base_parameters
        self.key_path = key_path
        self.api_version = api_version
        self.dapp_secret = dapp_secret
        self.action_auth_login = 'hep.auth.login'
        self.action_auth_pay = "hep.pay.order"
        self.action_auth_proof = "hep.proof.submit"
        self.chain_id = chain_id
        self.dapp_id = dapp_id

    def get_default_trust_oracle(self):
        """Retrieve the public key of default trust oracle from hep node
        
        :rtype: str
        :return: The public key of default trust oracle
        """
        params = {}
        sign_data = self.generate_sign_data(params)
        hmac_data = self.sign_hmac(sign_data)
        hmac_data['api_version'] = self.api_version
        hmac_data['oracle_id'] = 'default'
        res = self.api_client.rest_oracles_read(**hmac_data)
        return [res.public_key]

    def generate_sign_data(self, data):
        sign_data = copy.deepcopy(data)
        sign_data.update(self.base_parameters)
        # dynamic fields
        sign_data['ts'] = int(datetime.datetime.now().timestamp())
        sign_data['nonce'] = uuid.uuid4().hex
        return sign_data

    def sign_hmac(self, data, dapp_signature_method='HMAC-MD5'):
        dapp_signature = utils.sign_hmac(data, self.dapp_secret, signature_method=dapp_signature_method)
        data['dapp_signature'] = dapp_signature
        data['dapp_signature_method'] = dapp_signature_method
        return data

    def concat_signature(self, r, s):
        if r.startswith('0x'):
            r = r[2:]
        if len(r) < 64:
            r = '0' * (64 - len(r)) + r
        if s.startswith('0x'):
            s = s[2:]
        if len(s) < 64:
            s = '0' * (64 - len(s)) + s
        return '0x' + r + s

    def sign_secp256r1(self, data):
        sign_string = utils.generate_signature_base_string(data, '&')
        r, s = utils.sign_secp256r1(sign_string, self.key_path)
        signature = self.concat_signature(r, s)
        data['signature'] = signature
        data['sign_type'] = 'secp256r1'
        return data

    def validate_r1_data(self, data):
        signed_message = utils.generate_signature_base_string(data, "&")
        r, s = utils.split_signature(data.get('signature'))
        valid_public_keys = self.get_default_trust_oracle()
        return utils.validate_secp256r1_signature(r, s, signed_message, valid_public_keys)



# coding: utf-8

"""
    HEP REST API

    The REST API for HEP protocol  # noqa: E501

    OpenAPI spec version: v1
    Contact: xiawu@zeuux.org
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six
from hep_rest_api.models.order_item import OrderItem  # noqa: F401,E501


class Order(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'description': 'str',
        'price_currency': 'str',
        'total_price': 'str',
        'order_number': 'str',
        'seller': 'str',
        'customer': 'str',
        'broker': 'str',
        'order_items': 'list[OrderItem]',
        'chain_txid': 'str'
    }

    attribute_map = {
        'description': 'description',
        'price_currency': 'price_currency',
        'total_price': 'total_price',
        'order_number': 'order_number',
        'seller': 'seller',
        'customer': 'customer',
        'broker': 'broker',
        'order_items': 'order_items',
        'chain_txid': 'chain_txid'
    }

    def __init__(self, description=None, price_currency=None, total_price=None, order_number=None, seller=None, customer=None, broker=None, order_items=None, chain_txid=None):  # noqa: E501
        """Order - a model defined in Swagger"""  # noqa: E501
        self._description = None
        self._price_currency = None
        self._total_price = None
        self._order_number = None
        self._seller = None
        self._customer = None
        self._broker = None
        self._order_items = None
        self._chain_txid = None
        self.discriminator = None
        self.description = description
        self.price_currency = price_currency
        self.total_price = total_price
        self.order_number = order_number
        self.seller = seller
        self.customer = customer
        if broker is not None:
            self.broker = broker
        self.order_items = order_items
        if chain_txid is not None:
            self.chain_txid = chain_txid

    @property
    def description(self):
        """Gets the description of this Order.  # noqa: E501

        The order description  # noqa: E501

        :return: The description of this Order.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this Order.

        The order description  # noqa: E501

        :param description: The description of this Order.  # noqa: E501
        :type: str
        """
        if description is None:
            raise ValueError("Invalid value for `description`, must not be `None`")  # noqa: E501

        self._description = description

    @property
    def price_currency(self):
        """Gets the price_currency of this Order.  # noqa: E501

        The symbol of fiat or digital token, such as USD, RMB, NEW,BTC,ETH.  # noqa: E501

        :return: The price_currency of this Order.  # noqa: E501
        :rtype: str
        """
        return self._price_currency

    @price_currency.setter
    def price_currency(self, price_currency):
        """Sets the price_currency of this Order.

        The symbol of fiat or digital token, such as USD, RMB, NEW,BTC,ETH.  # noqa: E501

        :param price_currency: The price_currency of this Order.  # noqa: E501
        :type: str
        """
        if price_currency is None:
            raise ValueError("Invalid value for `price_currency`, must not be `None`")  # noqa: E501

        self._price_currency = price_currency

    @property
    def total_price(self):
        """Gets the total_price of this Order.  # noqa: E501

        The amount of fiat or digital token, unit is the minimum unit of given fiat or digital token.  # noqa: E501

        :return: The total_price of this Order.  # noqa: E501
        :rtype: str
        """
        return self._total_price

    @total_price.setter
    def total_price(self, total_price):
        """Sets the total_price of this Order.

        The amount of fiat or digital token, unit is the minimum unit of given fiat or digital token.  # noqa: E501

        :param total_price: The total_price of this Order.  # noqa: E501
        :type: str
        """
        if total_price is None:
            raise ValueError("Invalid value for `total_price`, must not be `None`")  # noqa: E501

        self._total_price = total_price

    @property
    def order_number(self):
        """Gets the order_number of this Order.  # noqa: E501

        The order number  # noqa: E501

        :return: The order_number of this Order.  # noqa: E501
        :rtype: str
        """
        return self._order_number

    @order_number.setter
    def order_number(self, order_number):
        """Sets the order_number of this Order.

        The order number  # noqa: E501

        :param order_number: The order_number of this Order.  # noqa: E501
        :type: str
        """
        if order_number is None:
            raise ValueError("Invalid value for `order_number`, must not be `None`")  # noqa: E501

        self._order_number = order_number

    @property
    def seller(self):
        """Gets the seller of this Order.  # noqa: E501

        The seller's NewID  # noqa: E501

        :return: The seller of this Order.  # noqa: E501
        :rtype: str
        """
        return self._seller

    @seller.setter
    def seller(self, seller):
        """Sets the seller of this Order.

        The seller's NewID  # noqa: E501

        :param seller: The seller of this Order.  # noqa: E501
        :type: str
        """
        if seller is None:
            raise ValueError("Invalid value for `seller`, must not be `None`")  # noqa: E501

        self._seller = seller

    @property
    def customer(self):
        """Gets the customer of this Order.  # noqa: E501

        The customer's NewID  # noqa: E501

        :return: The customer of this Order.  # noqa: E501
        :rtype: str
        """
        return self._customer

    @customer.setter
    def customer(self, customer):
        """Sets the customer of this Order.

        The customer's NewID  # noqa: E501

        :param customer: The customer of this Order.  # noqa: E501
        :type: str
        """
        if customer is None:
            raise ValueError("Invalid value for `customer`, must not be `None`")  # noqa: E501

        self._customer = customer

    @property
    def broker(self):
        """Gets the broker of this Order.  # noqa: E501

        The broker's NewID. optional.  # noqa: E501

        :return: The broker of this Order.  # noqa: E501
        :rtype: str
        """
        return self._broker

    @broker.setter
    def broker(self, broker):
        """Sets the broker of this Order.

        The broker's NewID. optional.  # noqa: E501

        :param broker: The broker of this Order.  # noqa: E501
        :type: str
        """

        self._broker = broker

    @property
    def order_items(self):
        """Gets the order_items of this Order.  # noqa: E501


        :return: The order_items of this Order.  # noqa: E501
        :rtype: list[OrderItem]
        """
        return self._order_items

    @order_items.setter
    def order_items(self, order_items):
        """Sets the order_items of this Order.


        :param order_items: The order_items of this Order.  # noqa: E501
        :type: list[OrderItem]
        """
        if order_items is None:
            raise ValueError("Invalid value for `order_items`, must not be `None`")  # noqa: E501

        self._order_items = order_items

    @property
    def chain_txid(self):
        """Gets the chain_txid of this Order.  # noqa: E501

        The chain txid  # noqa: E501

        :return: The chain_txid of this Order.  # noqa: E501
        :rtype: str
        """
        return self._chain_txid

    @chain_txid.setter
    def chain_txid(self, chain_txid):
        """Sets the chain_txid of this Order.

        The chain txid  # noqa: E501

        :param chain_txid: The chain_txid of this Order.  # noqa: E501
        :type: str
        """

        self._chain_txid = chain_txid

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Order, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Order):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



import sys
import uuid
import os
# set the path
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.insert(0, os.path.join(BASE_DIR, ""))

import hep_rest_api
from hep_rest_api.scenarios.auth import AuthHelper
from hep_rest_api.scenarios.pay import PayHelper
from hep_rest_api.scenarios.proof import ProofHelper
from hep_rest_api.scenarios.proof import OrderProof
from hep_rest_api.scenarios.proof import Order


PRIVATE_KEY_PATH = "/Users/erhu/pony/priv"
HEP_KEY = "71ffeae1a9a2402c944d84c54f8ffddc"
HEP_SECRET = "2d66e7f3dd4445dbb6791b56fadcd2dc"
HEP_PROTOCOL = "HEP"
HEP_PROTOCOL_VERSION = "1.0"
HEP_HOST = "http://hep.newtonproject.dev.diynova.com"
HEP_ID = "75098291f88343b9836118546f375a9f"
TEST_NEWID = "NEWID1acGJchbdZy74f3dTQxfZd6kkztfxzUgLtUyTvUtU21U4RaS72XY"
base_parameters = {
    'dapp_key': HEP_KEY,
    'protocol': HEP_PROTOCOL,
    'version': HEP_PROTOCOL_VERSION,
    'os': sys.platform,
    'language': 'en'
}
chain_id = 1002
key_path = PRIVATE_KEY_PATH
configuration = hep_rest_api.api_client.Configuration()
configuration.host = HEP_HOST
api_client = hep_rest_api.RestApi(hep_rest_api.ApiClient(configuration))
payment_data = {"dapp_id": "75098291f88343b9836118546f375a9f", "nonce": "92ac0a98ba1742c48eb8e91100077817", "sign_type": "secp256r1", "signature": "0xee47bd99d1807b7a14bb36c9e9fe641753f13e3c5d77610121edd3e125ddf78d22a56c98de8012d185b6de0da27aaef4101d3fe4beb5cf3038bc134d73012f48", "ts": "1561121020", "txid": "0xd7ab1ddcad52efd96298610030c985083cb6639b3f0f07c86f51ea7845a61237", "uuid": "b8e89a114b104a8d83e0266bbc5a55a1"}
login_data = {"action": "hep.auth.login", "scope": 1, "expired": 1561129670, "memo": "default", "uuid": "682749f075b74702b8c41c9a75862b0a", "dapp_id": "75098291f88343b9836118546f375a9f", "dapp_key": "71ffeae1a9a2402c944d84c54f8ffddc", "protocol":"HEP", "version": "1.0", "os": "darwin", "language": "en", "ts": 1561129370, "nonce": "ae892ffdf7ee430a992f29b5760d4e69", "dapp_signature": "046fbe22f261c5e5b62b37679b8f5a99", "dapp_signature_method": "HMAC-MD5", "signature": "0xf33c0e088f1cb64a8d0845a62333500d73b227c242e36dcf902da4243e515472dd2de175443b2918c52fbd6abbcbbace98a8e75540a93e3d29628096437f09e7", "sign_type": "secp256r1"}
login_profile = {"address": "NEW17xYWcvn5cp7rgYubVeenHZLGDJ5JtJapUPm", "avatar": "", "cellphone": "18888888881", "country_code": "86", "invite_code": "XLFFKD", "name":"不上班", "newid": "NEWID1acGJchbdZy74f3dTQxfZd6kkztfxzUgLtUyTvUtU21U4RaS72XY", "sign_type": "secp256r1", "signature": "0x8132dc49de81e4a1b85aaaa142fce6502f1093d495004240c2fee7bbf39e7ddb5c4565537ebb9b50312b6c34d267724f6567aa473dc85c9b268efc8f46f2235d", "uuid": "c11494156640416393a8d7c9926ffa87"}

def _get_api_client():
    configuration = hep_rest_api.api_client.Configuration()
    configuration.host = HEP_HOST
    api_instance = hep_rest_api.RestApi(hep_rest_api.ApiClient(configuration))
    return api_instance


def test_auth_login():
    api_client = _get_api_client()
    auth_helper = AuthHelper(api_client, base_parameters, HEP_ID, HEP_SECRET, key_path, chain_id=chain_id)
    auth_resposne = auth_helper.generate_auth_request(uuid=uuid.uuid4().hex)
    qr_str = auth_helper.generate_qrcode_string(auth_resposne.auth_hash)
    print(qr_str)
    is_valid = auth_helper.validate_auth_callback(login_profile)
    print(is_valid)


def test_auth_pay():
    api_client = _get_api_client()
    pay_helper = PayHelper(api_client, base_parameters, HEP_ID, HEP_SECRET, key_path, chain_id=chain_id)
    pay_response = pay_helper.generate_pay_request(uuid.uuid4().hex, "NEW", 12, "description", "NEWID1acGJchbdZy74f3dTQxfZd6kkztfxzUgLtUyTvUtU21U4RaS72XY", "NEWID1acGJchbdZy74f3dTQxfZd6kkztfxzUgLtUyTvUtU21U4RaS72XY", "NEWID1acGJchbdZy74f3dTQxfZd6kkztfxzUgLtUyTvUtU21U4RaS72XY", uuid=uuid.uuid4().hex)
    print(pay_response.pay_hash)
    pay_qr_str = pay_helper.generate_qrcode_string(pay_response.pay_hash)
    print(pay_qr_str)
    is_valid = pay_helper.validate_pay_callback(payment_data)
    print(is_valid)
    pay_data = {'api_version': '1', 'dapp_key': '71ffeae1a9a2402c944d84c54f8ffddc', 'protocol': 'HEP', 'version': '1.0', 'os': 'darwin', 'language': 'en', 'ts': 1561189219, 'nonce': '7fd6675608ba49d48c4e5c2f7e87bd8f', 'dapp_signature': 'beccf75e3f0aad44e25e8a50c37f7476', 'dapp_signature_method': 'HMAC-MD5', 'txid': '0xd7ab1ddcad52efd96298610030c985083cb6639b3f0f07c86f51ea7845a61237'}
    response = pay_helper.get_confirmed_transaction(pay_data['txid'])
    print(response)


def test_auth_proof():
    api_client = _get_api_client()
    order = Order( uuid.uuid4().hex, "deacription", "30", "CNY", TEST_NEWID, TEST_NEWID)
    order.add_order_item(uuid.uuid4().hex, 1, "10", "CNY", "pingguo", uuid.uuid4().hex)
    order.add_order_item(uuid.uuid4().hex, 2, "20", "CNY", "xiangjiao", uuid.uuid4().hex)
    order_content = OrderProof("30", "CNY", TEST_NEWID)
    order_content.add_order(order.to_dict())
    print(order_content.to_dict())
    proof_helper = ProofHelper(api_client, base_parameters, HEP_ID, HEP_SECRET, key_path, chain_id=chain_id)
    oracles = proof_helper.get_default_trust_oracle()
    print(oracles)
    proof_resposne = proof_helper.generate_proof_request(order_content.to_dict(), uuid=uuid.uuid4().hex)
    proof_str = proof_helper.generate_qrcode_string(proof_resposne.proof_hash)
    print(proof_str)
    proof_hashes = [
        proof_resposne.proof_hash
    ]
    response = proof_helper.get_status_of_proofs(proof_hashes)
    print(response)


if __name__ == '__main__':
    # print("test--login--")
    # test_auth_login()
    # print("test---pay---")
    # test_auth_pay()
    print("test---proof---")
    test_auth_proof()



# coding: utf-8

"""
    HEP REST API

    The REST API for HEP protocol  # noqa: E501

    OpenAPI spec version: v1
    Contact: xiawu@zeuux.org
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six


class NewchainTransactionResponse(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'txid': 'str',
        'order_number': 'str',
        'from_address': 'str',
        'to_address': 'str',
        'value': 'str',
        'block_height': 'int',
        'status': 'int'
    }

    attribute_map = {
        'txid': 'txid',
        'order_number': 'order_number',
        'from_address': 'from_address',
        'to_address': 'to_address',
        'value': 'value',
        'block_height': 'block_height',
        'status': 'status'
    }

    def __init__(self, txid=None, order_number=None, from_address=None, to_address=None, value=None, block_height=None, status=None):  # noqa: E501
        """NewchainTransactionResponse - a model defined in Swagger"""  # noqa: E501
        self._txid = None
        self._order_number = None
        self._from_address = None
        self._to_address = None
        self._value = None
        self._block_height = None
        self._status = None
        self.discriminator = None
        self.txid = txid
        self.order_number = order_number
        self.from_address = from_address
        self.to_address = to_address
        self.value = value
        self.block_height = block_height
        self.status = status

    @property
    def txid(self):
        """Gets the txid of this NewchainTransactionResponse.  # noqa: E501

        The transaction ID  # noqa: E501

        :return: The txid of this NewchainTransactionResponse.  # noqa: E501
        :rtype: str
        """
        return self._txid

    @txid.setter
    def txid(self, txid):
        """Sets the txid of this NewchainTransactionResponse.

        The transaction ID  # noqa: E501

        :param txid: The txid of this NewchainTransactionResponse.  # noqa: E501
        :type: str
        """
        if txid is None:
            raise ValueError("Invalid value for `txid`, must not be `None`")  # noqa: E501

        self._txid = txid

    @property
    def order_number(self):
        """Gets the order_number of this NewchainTransactionResponse.  # noqa: E501

        The order number  # noqa: E501

        :return: The order_number of this NewchainTransactionResponse.  # noqa: E501
        :rtype: str
        """
        return self._order_number

    @order_number.setter
    def order_number(self, order_number):
        """Sets the order_number of this NewchainTransactionResponse.

        The order number  # noqa: E501

        :param order_number: The order_number of this NewchainTransactionResponse.  # noqa: E501
        :type: str
        """
        if order_number is None:
            raise ValueError("Invalid value for `order_number`, must not be `None`")  # noqa: E501

        self._order_number = order_number

    @property
    def from_address(self):
        """Gets the from_address of this NewchainTransactionResponse.  # noqa: E501

        The from address  # noqa: E501

        :return: The from_address of this NewchainTransactionResponse.  # noqa: E501
        :rtype: str
        """
        return self._from_address

    @from_address.setter
    def from_address(self, from_address):
        """Sets the from_address of this NewchainTransactionResponse.

        The from address  # noqa: E501

        :param from_address: The from_address of this NewchainTransactionResponse.  # noqa: E501
        :type: str
        """
        if from_address is None:
            raise ValueError("Invalid value for `from_address`, must not be `None`")  # noqa: E501

        self._from_address = from_address

    @property
    def to_address(self):
        """Gets the to_address of this NewchainTransactionResponse.  # noqa: E501

        The from address  # noqa: E501

        :return: The to_address of this NewchainTransactionResponse.  # noqa: E501
        :rtype: str
        """
        return self._to_address

    @to_address.setter
    def to_address(self, to_address):
        """Sets the to_address of this NewchainTransactionResponse.

        The from address  # noqa: E501

        :param to_address: The to_address of this NewchainTransactionResponse.  # noqa: E501
        :type: str
        """
        if to_address is None:
            raise ValueError("Invalid value for `to_address`, must not be `None`")  # noqa: E501

        self._to_address = to_address

    @property
    def value(self):
        """Gets the value of this NewchainTransactionResponse.  # noqa: E501

        The transfer amount  # noqa: E501

        :return: The value of this NewchainTransactionResponse.  # noqa: E501
        :rtype: str
        """
        return self._value

    @value.setter
    def value(self, value):
        """Sets the value of this NewchainTransactionResponse.

        The transfer amount  # noqa: E501

        :param value: The value of this NewchainTransactionResponse.  # noqa: E501
        :type: str
        """
        if value is None:
            raise ValueError("Invalid value for `value`, must not be `None`")  # noqa: E501

        self._value = value

    @property
    def block_height(self):
        """Gets the block_height of this NewchainTransactionResponse.  # noqa: E501

        The block height  # noqa: E501

        :return: The block_height of this NewchainTransactionResponse.  # noqa: E501
        :rtype: int
        """
        return self._block_height

    @block_height.setter
    def block_height(self, block_height):
        """Sets the block_height of this NewchainTransactionResponse.

        The block height  # noqa: E501

        :param block_height: The block_height of this NewchainTransactionResponse.  # noqa: E501
        :type: int
        """
        if block_height is None:
            raise ValueError("Invalid value for `block_height`, must not be `None`")  # noqa: E501

        self._block_height = block_height

    @property
    def status(self):
        """Gets the status of this NewchainTransactionResponse.  # noqa: E501

        The status of transaction  # noqa: E501

        :return: The status of this NewchainTransactionResponse.  # noqa: E501
        :rtype: int
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this NewchainTransactionResponse.

        The status of transaction  # noqa: E501

        :param status: The status of this NewchainTransactionResponse.  # noqa: E501
        :type: int
        """
        if status is None:
            raise ValueError("Invalid value for `status`, must not be `None`")  # noqa: E501

        self._status = status

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(NewchainTransactionResponse, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, NewchainTransactionResponse):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



"""The utility of HEP REST SDK

"""

__copyright__ = """ Copyright (c) 2019 Newton Foundation. All rights reserved."""
__version__ = '1.0'
__author__ = 'xiawu@newtonproject.org'

import json
import hashlib
import collections
import base58
import binascii
import logging
from fastecdsa import curve, ecdsa, keys
from sha3 import keccak_256

logger = logging.getLogger(__name__)
SUPPORT_SIGNATURE_METHODS = ['HMAC-MD5', ]
PREFIX = 'NEW'


def generate_md5(data):
    return hashlib.md5(data.encode('utf-8')).hexdigest()


def generate_signature_base_string(data, joint):
    """Generate the concatenated base string for signature

    :param dict data: The data for signature
    :param str joint: The joint string for concatenation
    :rtype: str
    :return: The concatendated base string
    """
    ordered_data = collections.OrderedDict(sorted(data.items()))
    sign_string = ''
    sign_fields = ['dapp_signature_method', 'dapp_signature', 'signature', 'sign_type']
    n = 0
    for k, v in ordered_data.items():
        if k in sign_fields:
            continue
        if n != 0:
            sign_string += joint
        n += 1
        if isinstance(v, list) or isinstance(v, dict):
            sign_string += u'%s=%s' % (k, json.dumps(v, sort_keys=True, separators=(',', ':'), ensure_ascii=False))
        else:
            sign_string += u'%s=%s' % (k, v)
    return sign_string

def sign_hmac(data, secret, prefix='', use_urlencode=False, joint='&', signature_method='HMAC-MD5'):
    """Generate the HMAC signature string

    :param dict data: The data for signature
    :param str secret: The HMAC secret
    :param str prefix: The concatendated prefix
    :param bool use_urlencode: Whether use urlencode
    :param str joint: The joint string for concatenation
    :param str signature_method: The signature method
    :rtype: str
    :return: The HMAC signature string
    """

    if signature_method not in SUPPORT_SIGNATURE_METHODS:
        raise Exception('Unsupported signature method')
    sign_string = prefix
    sign_string += generate_signature_base_string(data, joint)
    sign_string += secret
    if use_urlencode:
        sign_string = urllib.quote_plus(sign_string)
    signed_string = generate_md5(sign_string)
    return signed_string

def generate_secp256r1_key_pairs(save_path):
    """Generate the key pairs of secp256r1
    
    :param str save_path: The save file path of private key
    :rtype:tuple
    :return: The hex string of public key
    """
    private_key, public_key = keys.gen_keypair(curve.P256)
    keys.export_key(private_key, curve.P256, filepath=save_path)
    x = str(hex(public_key.x))[2:]
    y = str(hex(public_key.y))[2:]
    if len(x) < 64:
        x = '0' * (64 - len(x)) + x
    if len(y) < 64:
        y = '0' * (64 - len(y)) + y
    public_key =  '0x' + x + y
    return public_key

def sign_secp256r1(message, private_key_path, hashfunc=keccak_256):
    """Sign the message by secp256r1
    
    :param str message: The message for sign
    :param str private_key_path: The file path of private key
    :param function hashfunc: The hash function
    :rtype: tuple
    :return: The r and s part of signature
    """
    private_key = keys.import_key(private_key_path)[0]
    r, s = ecdsa.sign(message, private_key, hashfunc=hashfunc)
    return str(hex(r)), str(hex(s))

def extract_secp256r1_public_keys(r, s, message):
    """Extract the public keys from r, s, message which generated by secp256r1

    :param str r: The r part of signature
    :param str s: The s part of signature
    :param str message: The message for signature
    :rtype: list
    :return: The public keys    
    """    
    public_key_point = keys.get_public_keys_from_sig((int(r, 16), int(s, 16)), message, curve.P256, hashfunc=keccak_256)
    public_key_list = []
    for i in public_key_point:
        x = str(hex(i.x)[2:])
        y = str(hex(i.y)[2:])
        if len(x) < 64:
            x = '0' * (64 - len(x)) + x
        if len(y) < 64:
            y = '0' * (64 - len(y)) + y
        public_key = '0x' + x + y
        public_key_list.append(public_key)
    return public_key_list

def validate_secp256r1_signature(r, s, message, valid_public_keys):
    """Validate the secp256r1 signature

    :param str r: The r part of signature
    :param str s: The s part of signature
    :param str message: The message for signature
    :param list valid_public_keys: The public key list for validation
    :rtype: bool
    :return: True if it is valid signature, otherwise False    
    """
    public_keys = extract_secp256r1_public_keys(r, s, message)
    logger.debug("public_keys:%s, valid_public_keys:%s" % (public_keys, valid_public_keys))
    for public_key in public_keys:
        if public_key in valid_public_keys:
            return True
    return False


def newid_encode_by_public_key(public_key, chain_id):
    """ generate newid by public key

    :param str public_key: The public key of newid
    :param int chain_id: The blockchain ID
    :rtype: str
    :return: The encoded newid
    """
    if public_key.startswith('0x'):
        public_key = public_key[2:]
    if len(public_key) < 64:
        public_key = '0' * (64 - len(public_key)) + public_key
    k = keccak_256()
    k.update(bytearray.fromhex(public_key))
    data = k.hexdigest()
    hex_chainID = hex(chain_id)[2:][-8:]
    if (len(hex_chainID) % 2) == 1:
        hex_chainID = '0' + hex_chainID
    num_sum = hex_chainID + data
    data = base58.b58encode_check(b'\0' + binascii.a2b_hex(num_sum))
    newid = PREFIX + 'ID' + data.decode()
    return newid


def validate_newid(r, s, message, newid, chain_id):
    """Validate the newid signature

    :param str r: The r part hex string for signature
    :param str s: The s part hex string for signature
    :param str message: The message which signed
    :param str newid: The user's newid
    :param int chain_id: The NewChain id. The dev environment is 1002, test is 1007, main is 1012
    :rtype: bool
    :return: True if it is valid signature, otherwise False
    """
    public_keys = extract_secp256r1_public_keys(r, s, message)
    for public_key in public_keys:
        temp = newid_encode_by_public_key(public_key, chain_id)
        if temp == newid:
            return True
    return False


def split_signature(signature):
    """Split the given signature

    :param signature:
    :rtype: str, str
    :return: The r part for signature, The s part for signature
    """
    if not signature:
        raise ValueError(
            "Missing the required parameter 'signature' when calling split_signature"
        )
    if signature.startswith("0x"):
        signature = signature[2:]
    sig_half_len = int(len(signature) / 2)
    r = '0x' + signature[: sig_half_len]
    s = '0x' + signature[sig_half_len:]
    return r, s



# coding: utf-8

"""
    HEP REST API

    The REST API for HEP protocol  # noqa: E501

    OpenAPI spec version: v1
    Contact: xiawu@zeuux.org
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six


class AuthCacheRequest(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'dapp_key': 'str',
        'protocol': 'str',
        'version': 'str',
        'ts': 'int',
        'nonce': 'str',
        'os': 'str',
        'language': 'str',
        'dapp_signature_method': 'str',
        'dapp_signature': 'str',
        'sign_type': 'str',
        'signature': 'str',
        'dapp_id': 'str',
        'uuid': 'str',
        'action': 'str',
        'scope': 'int',
        'expired': 'int',
        'memo': 'str'
    }

    attribute_map = {
        'dapp_key': 'dapp_key',
        'protocol': 'protocol',
        'version': 'version',
        'ts': 'ts',
        'nonce': 'nonce',
        'os': 'os',
        'language': 'language',
        'dapp_signature_method': 'dapp_signature_method',
        'dapp_signature': 'dapp_signature',
        'sign_type': 'sign_type',
        'signature': 'signature',
        'dapp_id': 'dapp_id',
        'uuid': 'uuid',
        'action': 'action',
        'scope': 'scope',
        'expired': 'expired',
        'memo': 'memo'
    }

    def __init__(self, dapp_key=None, protocol=None, version=None, ts=None, nonce=None, os=None, language=None, dapp_signature_method=None, dapp_signature=None, sign_type=None, signature=None, dapp_id=None, uuid=None, action=None, scope=None, expired=None, memo=None):  # noqa: E501
        """AuthCacheRequest - a model defined in Swagger"""  # noqa: E501
        self._dapp_key = None
        self._protocol = None
        self._version = None
        self._ts = None
        self._nonce = None
        self._os = None
        self._language = None
        self._dapp_signature_method = None
        self._dapp_signature = None
        self._sign_type = None
        self._signature = None
        self._dapp_id = None
        self._uuid = None
        self._action = None
        self._scope = None
        self._expired = None
        self._memo = None
        self.discriminator = None
        self.dapp_key = dapp_key
        self.protocol = protocol
        self.version = version
        self.ts = ts
        self.nonce = nonce
        self.os = os
        self.language = language
        self.dapp_signature_method = dapp_signature_method
        self.dapp_signature = dapp_signature
        self.sign_type = sign_type
        self.signature = signature
        self.dapp_id = dapp_id
        self.uuid = uuid
        self.action = action
        self.scope = scope
        self.expired = expired
        if memo is not None:
            self.memo = memo

    @property
    def dapp_key(self):
        """Gets the dapp_key of this AuthCacheRequest.  # noqa: E501

        The decentralized application access key  # noqa: E501

        :return: The dapp_key of this AuthCacheRequest.  # noqa: E501
        :rtype: str
        """
        return self._dapp_key

    @dapp_key.setter
    def dapp_key(self, dapp_key):
        """Sets the dapp_key of this AuthCacheRequest.

        The decentralized application access key  # noqa: E501

        :param dapp_key: The dapp_key of this AuthCacheRequest.  # noqa: E501
        :type: str
        """
        if dapp_key is None:
            raise ValueError("Invalid value for `dapp_key`, must not be `None`")  # noqa: E501

        self._dapp_key = dapp_key

    @property
    def protocol(self):
        """Gets the protocol of this AuthCacheRequest.  # noqa: E501

        The protocol name. default is 'HEP'.  # noqa: E501

        :return: The protocol of this AuthCacheRequest.  # noqa: E501
        :rtype: str
        """
        return self._protocol

    @protocol.setter
    def protocol(self, protocol):
        """Sets the protocol of this AuthCacheRequest.

        The protocol name. default is 'HEP'.  # noqa: E501

        :param protocol: The protocol of this AuthCacheRequest.  # noqa: E501
        :type: str
        """
        if protocol is None:
            raise ValueError("Invalid value for `protocol`, must not be `None`")  # noqa: E501

        self._protocol = protocol

    @property
    def version(self):
        """Gets the version of this AuthCacheRequest.  # noqa: E501

        The protocol version such as '1.0'  # noqa: E501

        :return: The version of this AuthCacheRequest.  # noqa: E501
        :rtype: str
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this AuthCacheRequest.

        The protocol version such as '1.0'  # noqa: E501

        :param version: The version of this AuthCacheRequest.  # noqa: E501
        :type: str
        """
        if version is None:
            raise ValueError("Invalid value for `version`, must not be `None`")  # noqa: E501

        self._version = version

    @property
    def ts(self):
        """Gets the ts of this AuthCacheRequest.  # noqa: E501

        The current timestamp  # noqa: E501

        :return: The ts of this AuthCacheRequest.  # noqa: E501
        :rtype: int
        """
        return self._ts

    @ts.setter
    def ts(self, ts):
        """Sets the ts of this AuthCacheRequest.

        The current timestamp  # noqa: E501

        :param ts: The ts of this AuthCacheRequest.  # noqa: E501
        :type: int
        """
        if ts is None:
            raise ValueError("Invalid value for `ts`, must not be `None`")  # noqa: E501

        self._ts = ts

    @property
    def nonce(self):
        """Gets the nonce of this AuthCacheRequest.  # noqa: E501

        The random string or auto-increment sequence  # noqa: E501

        :return: The nonce of this AuthCacheRequest.  # noqa: E501
        :rtype: str
        """
        return self._nonce

    @nonce.setter
    def nonce(self, nonce):
        """Sets the nonce of this AuthCacheRequest.

        The random string or auto-increment sequence  # noqa: E501

        :param nonce: The nonce of this AuthCacheRequest.  # noqa: E501
        :type: str
        """
        if nonce is None:
            raise ValueError("Invalid value for `nonce`, must not be `None`")  # noqa: E501

        self._nonce = nonce

    @property
    def os(self):
        """Gets the os of this AuthCacheRequest.  # noqa: E501

        The operating system of client such as ios, android, dweb,etc.  # noqa: E501

        :return: The os of this AuthCacheRequest.  # noqa: E501
        :rtype: str
        """
        return self._os

    @os.setter
    def os(self, os):
        """Sets the os of this AuthCacheRequest.

        The operating system of client such as ios, android, dweb,etc.  # noqa: E501

        :param os: The os of this AuthCacheRequest.  # noqa: E501
        :type: str
        """
        if os is None:
            raise ValueError("Invalid value for `os`, must not be `None`")  # noqa: E501

        self._os = os

    @property
    def language(self):
        """Gets the language of this AuthCacheRequest.  # noqa: E501

        The i18n language code such as zh, en, etc.  # noqa: E501

        :return: The language of this AuthCacheRequest.  # noqa: E501
        :rtype: str
        """
        return self._language

    @language.setter
    def language(self, language):
        """Sets the language of this AuthCacheRequest.

        The i18n language code such as zh, en, etc.  # noqa: E501

        :param language: The language of this AuthCacheRequest.  # noqa: E501
        :type: str
        """
        if language is None:
            raise ValueError("Invalid value for `language`, must not be `None`")  # noqa: E501

        self._language = language

    @property
    def dapp_signature_method(self):
        """Gets the dapp_signature_method of this AuthCacheRequest.  # noqa: E501

        The signature method used by dapp.  # noqa: E501

        :return: The dapp_signature_method of this AuthCacheRequest.  # noqa: E501
        :rtype: str
        """
        return self._dapp_signature_method

    @dapp_signature_method.setter
    def dapp_signature_method(self, dapp_signature_method):
        """Sets the dapp_signature_method of this AuthCacheRequest.

        The signature method used by dapp.  # noqa: E501

        :param dapp_signature_method: The dapp_signature_method of this AuthCacheRequest.  # noqa: E501
        :type: str
        """
        if dapp_signature_method is None:
            raise ValueError("Invalid value for `dapp_signature_method`, must not be `None`")  # noqa: E501

        self._dapp_signature_method = dapp_signature_method

    @property
    def dapp_signature(self):
        """Gets the dapp_signature of this AuthCacheRequest.  # noqa: E501

        The signature generated by dapp.  # noqa: E501

        :return: The dapp_signature of this AuthCacheRequest.  # noqa: E501
        :rtype: str
        """
        return self._dapp_signature

    @dapp_signature.setter
    def dapp_signature(self, dapp_signature):
        """Sets the dapp_signature of this AuthCacheRequest.

        The signature generated by dapp.  # noqa: E501

        :param dapp_signature: The dapp_signature of this AuthCacheRequest.  # noqa: E501
        :type: str
        """
        if dapp_signature is None:
            raise ValueError("Invalid value for `dapp_signature`, must not be `None`")  # noqa: E501

        self._dapp_signature = dapp_signature

    @property
    def sign_type(self):
        """Gets the sign_type of this AuthCacheRequest.  # noqa: E501

        The signature Type,aka cryptographic algorithm.  # noqa: E501

        :return: The sign_type of this AuthCacheRequest.  # noqa: E501
        :rtype: str
        """
        return self._sign_type

    @sign_type.setter
    def sign_type(self, sign_type):
        """Sets the sign_type of this AuthCacheRequest.

        The signature Type,aka cryptographic algorithm.  # noqa: E501

        :param sign_type: The sign_type of this AuthCacheRequest.  # noqa: E501
        :type: str
        """
        if sign_type is None:
            raise ValueError("Invalid value for `sign_type`, must not be `None`")  # noqa: E501

        self._sign_type = sign_type

    @property
    def signature(self):
        """Gets the signature of this AuthCacheRequest.  # noqa: E501

        The signature hex string by application owner. The exclude fields is [sign_type, signature, md5].  # noqa: E501

        :return: The signature of this AuthCacheRequest.  # noqa: E501
        :rtype: str
        """
        return self._signature

    @signature.setter
    def signature(self, signature):
        """Sets the signature of this AuthCacheRequest.

        The signature hex string by application owner. The exclude fields is [sign_type, signature, md5].  # noqa: E501

        :param signature: The signature of this AuthCacheRequest.  # noqa: E501
        :type: str
        """
        if signature is None:
            raise ValueError("Invalid value for `signature`, must not be `None`")  # noqa: E501

        self._signature = signature

    @property
    def dapp_id(self):
        """Gets the dapp_id of this AuthCacheRequest.  # noqa: E501

        The decentralized application ID  # noqa: E501

        :return: The dapp_id of this AuthCacheRequest.  # noqa: E501
        :rtype: str
        """
        return self._dapp_id

    @dapp_id.setter
    def dapp_id(self, dapp_id):
        """Sets the dapp_id of this AuthCacheRequest.

        The decentralized application ID  # noqa: E501

        :param dapp_id: The dapp_id of this AuthCacheRequest.  # noqa: E501
        :type: str
        """
        if dapp_id is None:
            raise ValueError("Invalid value for `dapp_id`, must not be `None`")  # noqa: E501

        self._dapp_id = dapp_id

    @property
    def uuid(self):
        """Gets the uuid of this AuthCacheRequest.  # noqa: E501

        The request uuid  # noqa: E501

        :return: The uuid of this AuthCacheRequest.  # noqa: E501
        :rtype: str
        """
        return self._uuid

    @uuid.setter
    def uuid(self, uuid):
        """Sets the uuid of this AuthCacheRequest.

        The request uuid  # noqa: E501

        :param uuid: The uuid of this AuthCacheRequest.  # noqa: E501
        :type: str
        """
        if uuid is None:
            raise ValueError("Invalid value for `uuid`, must not be `None`")  # noqa: E501

        self._uuid = uuid

    @property
    def action(self):
        """Gets the action of this AuthCacheRequest.  # noqa: E501

        The action name which value is 'hep.auth.login'.  # noqa: E501

        :return: The action of this AuthCacheRequest.  # noqa: E501
        :rtype: str
        """
        return self._action

    @action.setter
    def action(self, action):
        """Sets the action of this AuthCacheRequest.

        The action name which value is 'hep.auth.login'.  # noqa: E501

        :param action: The action of this AuthCacheRequest.  # noqa: E501
        :type: str
        """
        if action is None:
            raise ValueError("Invalid value for `action`, must not be `None`")  # noqa: E501

        self._action = action

    @property
    def scope(self):
        """Gets the scope of this AuthCacheRequest.  # noqa: E501

        The request profile type. 1: base profile including name,head,newid; 2: advance profile including cellphone.  # noqa: E501

        :return: The scope of this AuthCacheRequest.  # noqa: E501
        :rtype: int
        """
        return self._scope

    @scope.setter
    def scope(self, scope):
        """Sets the scope of this AuthCacheRequest.

        The request profile type. 1: base profile including name,head,newid; 2: advance profile including cellphone.  # noqa: E501

        :param scope: The scope of this AuthCacheRequest.  # noqa: E501
        :type: int
        """
        if scope is None:
            raise ValueError("Invalid value for `scope`, must not be `None`")  # noqa: E501

        self._scope = scope

    @property
    def expired(self):
        """Gets the expired of this AuthCacheRequest.  # noqa: E501

        The expired timestamp  # noqa: E501

        :return: The expired of this AuthCacheRequest.  # noqa: E501
        :rtype: int
        """
        return self._expired

    @expired.setter
    def expired(self, expired):
        """Sets the expired of this AuthCacheRequest.

        The expired timestamp  # noqa: E501

        :param expired: The expired of this AuthCacheRequest.  # noqa: E501
        :type: int
        """
        if expired is None:
            raise ValueError("Invalid value for `expired`, must not be `None`")  # noqa: E501

        self._expired = expired

    @property
    def memo(self):
        """Gets the memo of this AuthCacheRequest.  # noqa: E501

        The login Memo,optional.  # noqa: E501

        :return: The memo of this AuthCacheRequest.  # noqa: E501
        :rtype: str
        """
        return self._memo

    @memo.setter
    def memo(self, memo):
        """Sets the memo of this AuthCacheRequest.

        The login Memo,optional.  # noqa: E501

        :param memo: The memo of this AuthCacheRequest.  # noqa: E501
        :type: str
        """

        self._memo = memo

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(AuthCacheRequest, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, AuthCacheRequest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    HEP REST API

    The REST API for HEP protocol  # noqa: E501

    OpenAPI spec version: v1
    Contact: xiawu@zeuux.org
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import unittest

import hep_rest_api
from models.receipt_field import ReceiptField  # noqa: E501
from hep_rest_api.rest import ApiException


class TestReceiptField(unittest.TestCase):
    """ReceiptField unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testReceiptField(self):
        """Test ReceiptField"""
        # FIXME: construct object with mandatory attributes with example values
        # model = hep_rest_api.models.receipt_field.ReceiptField()  # noqa: E501
        pass


if __name__ == '__main__':
    unittest.main()
